<?php
@include_once __DIR__ . "/vendor/autoload.php";

use Kirby\Cms\Blocks;
use Kirby\Cms\Page;
use Kirby\Toolkit\A;
use LavenderBits\Squaby\IssueService;
use LavenderBits\Squaby\Mail\TemplateService;
use LavenderBits\Squaby\Newsletter;
use LavenderBits\Squaby\SignupController;
use LavenderBits\Squaby\Subscriber;
use LavenderBits\Squaby\TrackingController;

Kirby::plugin("lavender-bits/squaby", [
    "blueprints" => [
        "pages/squaby-newsletter" =>
            __DIR__ . "/blueprints/pages/newsletter.yml",
        "pages/squaby-subscription" =>
            __DIR__ . "/blueprints/pages/subscription.yml",
    ],
    "snippets" => [
        "squaby-js" => __DIR__ . "/snippets/js.php",
    ],
    "routes" => A::merge(
        SignupController::routes(),
        TrackingController::routes()
    ),
    "pageModels" => [
        "squaby-newsletter" => Newsletter::class,
        "squaby-subscription" => Subscriber::class,
    ],
    "commands" => [
        "squaby:schedule" => require_once __DIR__ . "/commands/schedule.php",
        "squaby:deliver" => require_once __DIR__ . "/commands/deliver.php",
        "squaby:test" => [
            "description" => "Whatever Squaby is testing atm",
            "args" => [],
            "command" => static function ($cli): void {
                $issue = page("newsletter/new-issue-yay");
                /** @var Subscriber $subscriber */
                $subscriber = page("newsletter/skye");
                $link = $subscriber->createUniqueLink($issue);
                $cli->out("Link is: " . $link);
            },
        ],
    ],
    "siteMethods" => [
        "getOpensCountForIssue" => function (string $issueId): int {
            return IssueService::getOpensCountForIssue($issueId);
        },

        "getSendsCountForIssue" => function (string $issueId): int {
            return IssueService::getSendsCountForIssue($issueId);
        },

        "getSendToOpenRatioForIssue" => function (string $issueId): int {
            return IssueService::getSendToOpenRatioForIssue($issueId);
        },

        "currentSubscriber" => function (): Subscriber|false {
            $secret = get("sid");
            $newsletterId = get("nsl");
            if (
                !$secret ||
                empty($secret) ||
                !$newsletterId ||
                empty($newsletterId)
            ) {
                return false;
            }

            $newsletterId = urldecode($newsletterId);
            $newsletter = page($newsletterId);
            if (!$newsletter) {
                return false;
            }

            /** @var Subscriber | false $subscriber */
            $subscriber = false;
            $subscriber = $newsletter
                ->childrenAndDrafts()
                ->findBy("secret", $secret);
            if (!$subscriber) {
                return false;
            }
            return $subscriber;
        },

        "emailBlocks" => function (Blocks $blocks): string {
            return TemplateService::renderBlocks($blocks);
        },
    ],
]);
