<?php

namespace LavenderBits\Squaby;

/**
 * Provides utilities around issues and database entries about those issues.
 * Has to be its own static class as we can not expect Issues to be of any
 * specific model.
 */
class IssueService
{
    /**
     * Get the total number of opens tracked by the tracking pixel
     *
     * @param string $issueId The page ID (not uuid) of the issue
     *
     * @return int  The number of unique opens
     */
    public static function getOpensCountForIssue(string $issueId): int
    {
        $db = DBService::getDbConnection();
        return $db
            ->table("tracking")
            ->where([
                "issue_id" => strtolower(urlencode($issueId)),
                "opened" => 1,
            ])
            ->count();
    }

    /**
     * Get the total number of times an issue was sent to unique addresses
     *
     * @param string $issueId The page ID (not uuid) of the issue
     *
     * @return int  The number of unique sends
     */
    public static function getSendsCountForIssue(string $issueId): int
    {
        $db = DBService::getDbConnection();
        return $db
            ->table("tracking")
            ->where([
                "issue_id" => strtolower(urlencode($issueId)),
                "opened" => 1,
            ])
            ->count();
    }

    /**
     * Get the ratio of send to opens as a percentage
     *
     * @param string $issueId The page ID (not uuid) of the issue
     *
     * @return int  The ratio of sends/opens as a percentage (0-100)
     */
    public static function getSendToOpenRatioForIssue(string $issueId): int
    {
        return (self::getOpensCountForIssue($issueId) /
            self::getSendsCountForIssue($issueId)) *
            100;
    }
}
