<?php

namespace LavenderBits\Squaby;

use Kirby\Cms\Collection;
use Kirby\Cms\Page;
use Kirby\Toolkit\Date;

class Newsletter extends Page
{
    /**
     * Creates a new subscriber for a given email address
     * Automatically triggers the sending of a confirmation mail
     *
     * @param string $email The subscriber's email address
     *
     * @return bool  True if the confirmation mail was sent successfully
     */
    public function addSubscriber(string $email): bool
    {
        if ($this->childrenAndDrafts()->findBy("email", strtolower($email))) {
            // Handle people that are already subscribed
            // TODO: Maybe send a new confirmation e-mail if they aren't confirmed yet?
            return true;
        }

        return kirby()->impersonate("kirby", function () use (
            &$subscriber,
            $email
        ) {
            $subscriber = $this->createChild([
                "slug" => uuid(),
                "template" => "squaby-subscription", //TODO: Make variable
                "content" => [
                    "title" => $email,
                    "email" => $email,
                    "secret" => uuid(),
                    "date" => Date::now(),
                    "consent" => false,
                    "activeSubscriber" => false,
                ],
            ]);
            // TODO: Review if this is smart. We probably have use-cases where we want to add a subscriber manually without triggering this message
            return $subscriber->sendConfirmationMail();
        });
    }

    /**
     * Returns a collection of all active subscribers
     *
     * @return Collection  A Collection of LavenderBits\Squaby\Subscriber
     */
    public function getActiveSubscribers(): Collection
    {
        return $this->childrenAndDrafts()->filterBy("activeSubscriber", true);
    }
}
