<?php
namespace LavenderBits\Squaby;

use Error;
use Kirby\Cms\Page;
use Kirby\Database\Database;
use Kirby\Http\Response;

class TrackingController
{
    /**
     * Returns the routes published by this controller
     *
     * @return array
     */
    public static function routes(): array
    {
        return [
            [
                "pattern" => "/newsletter-api/pixel/(:any)/(:any)",
                "method" => "GET",
                "action" => function ($issueId, $unique) {
                    return TrackingController::track($issueId, $unique);
                },
            ],
        ];
    }

    /**
     * Track newsletter views
     *
     * @param string $issueId ID of the issue being viewed
     * @param string $uniqueKey Unique key of the specific email that was sent
     *
     * @return Response  Response containing a transparent 1px by 1px PNG
     */
    public static function track(string $issueId, string $uniqueKey): Response
    {
        try {
            $db = DBService::getDbConnection();
            $db->table("tracking")->update(
                [
                    "opened" => 1,
                ],
                [
                    "id" => $uniqueKey,
                    "issue_id" => strtolower(urldecode($issueId)),
                ]
            );
        } catch (Error $e) {
            // TODO: Handle better
        }
        return Response::file(kirby()->root("plugins") . "/squaby/track.png");
    }
}
