import NewsletterAPI from "../API/NewsletterAPI";
import Helper from "../helper";

export default class SignupField {
  submitButton;
  emailField;
  errorField;
  redirect;
  successElement;
  newsletter;
  api;
  isSubmitting = false;

  constructor(
    emailField,
    submitButton,
    newsletter,
    errorField = undefined,
    redirect = undefined,
    successElement = undefined,
  ) {
    this.emailField = emailField;
    this.submitButton = submitButton;
    this.errorField = errorField;
    this.successElement = successElement;
    this.redirect = redirect;
    this.newsletter = newsletter;
    this.api = new NewsletterAPI(newsletter);

    // Mount events
    this.submitButton.addEventListener("click", async (e) => {
      e.preventDefault();
      if (this.isSubmitting) {
        return;
      }
      if (!this.validateEmail()) {
        if (this.errorField) {
          this.errorField.innerText = "Invalid eMail address";
        }
        return;
      }
      this.isSubmitting = true;
      const res = await this.api.registerForNewsletter(this.emailField.value);
      if (res.success) {
        console.log(this.successElement);
        if (this.successElement) {
          this.successElement.style.removeProperty("display");
          console.log(this.successElement);
        }
        if (this.redirect) {
          document.location.href = this.redirect;
        }
      } else {
        console.error(res.error);
        if (this.errorField) {
          this.errorField.innerText = res.error;
        }
      }
      this.isSubmitting = false;
    });
  }

  startLiveValidation() {
    this.emailField.addEventListener("input", (e) => {
      if (Helper.validateEmail(this.emailField.value)) {
        this.emailField.classList.remove("has-errors");
        if (this.errorField) {
          this.errorField.innerText = "";
        }
      }
    });
  }

  validateEmail() {
    this.emailField.classList.remove("has-errors");
    if (!Helper.validateEmail(this.emailField.value)) {
      this.emailField.classList.add("has-errors");
      this.startLiveValidation();
      return false;
    }
    return true;
  }
}
