import SignupField from "./elements/SignupField";

document.addEventListener("DOMContentLoaded", (e) => {
  // Mount different types of components
  for (const signupField of document.querySelectorAll(
    '[data-role="newsletter-signup"]',
  )) {
    mountSignupField(signupField);
  }
});

/**
 * Mounts a signup field
 * @param {Element} element - The parent element. Contains the email and error field, as well as the submit button
 */
function mountSignupField(el) {
  // Newsletter Configuration
  const newsletter = el.dataset.newsletter;
  if (!newsletter) {
    throw new Error(
      `Newsletter ID (data-newsletter="...") missing in signup field ${el}`,
    );
  }

  // Optional behavior config
  const successElement = el.querySelector('[data-role="newsletter-success"]');
  const redirect = el.dataset.redirect;

  // UI elements
  const button = el.querySelector('[data-role="newsletter-submit"]');
  const email = el.querySelector('[data-role="newsletter-email"]');
  const error = el.querySelector('[data-role="newsletter-error"]');
  if (!email) {
    throw new Error(
      `Email field ([data-role="email"]) missing in signup field ${el}`,
    );
  }
  if (!button) {
    throw new Error(
      `Signup button ([data-role="submit"]) not found in signup field ${el}`,
    );
  }

  const element = new SignupField(
    email,
    button,
    newsletter,
    error,
    redirect,
    successElement,
  );
}
